package de.juplo.yourshouter;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.stereotype.Service;


/**
 * Specialized implementation of {@link AuthenticationEntryPoint}, that
 * redirects to the social sign-in-page, to let the user decide to sign in or
 * not.
 *
 * @author Kai Moritz
 */
@Service
public class SocialAuthenticationEntryPoint implements AuthenticationEntryPoint
{
  private static final Logger LOG =
      LoggerFactory.getLogger(SocialAuthenticationEntryPoint.class);

  public final static String REDIRECT_ATTRIBUTE =
      SocialAuthenticationEntryPoint.class.getCanonicalName() + ".REDIRECT";


  /**
   * {@inheritDoc}
   *
   * To commence the sign-in through the Graph-API, we have to redirect
   * to our already implemented sign-in-page.
   * <p>
   * We store the originally requested page in the {@link HttpSession}, to be
   * redirect back to that page after a successful authentication in
   * {@link SpringSecuritySignInAdapter}.
   * <p>
   * Only the first request of a ressource, that requires authentication, will
   * trigger the redirect to the sing-in-page.
   *
   * @see SpringSecuritySignInAdapter
   */
  @Override
  public void commence(
      HttpServletRequest request,
      HttpServletResponse response,
      AuthenticationException exception
      )
      throws
        IOException,
        ServletException
  {
    HttpSession session = request.getSession();
    if (session.getAttribute(REDIRECT_ATTRIBUTE) == null)
    {
      LOG.info(
          "redirecting unauthenticated request to {}",
          request.getRequestURI()
          );
      StringBuffer url = request.getRequestURL();
      if (request.getQueryString() != null)
      {
        url.append('?');
        url.append(request.getQueryString());
      }
      session.setAttribute(REDIRECT_ATTRIBUTE, url.toString());
      response.sendRedirect("/signin.html");
    }
    else
    {
      LOG.info(
          "redirect to sign-in already in progress, forbidding access to {}",
          request.getRequestURI()
          );
      response.sendError(HttpServletResponse.SC_FORBIDDEN);
    }
  }
}
